import { url } from '../../App';

import React, { useContext } from 'react';
import { useDispatch, useSelector } from "react-redux"
import { useHistory } from 'react-router-dom';

import { CardHeader, Avatar, Typography, Button, Box, Divider } from '@mui/material';
import { UpdateProfile } from '../../App';
import { actionFullUpdateProfile } from '../../redux/thunks';

import './style.scss'


// Карточки блока с рекомендациями
// export function RecommendedCard({ data }) {
function RecommendedCard({ data }) {
    // console.log('data: ', data)
    const history = useHistory()
    const dispatch = useDispatch()

    // контекст обновления профиля
    const [updateProfile, setUpdateProfile] = useContext(UpdateProfile)
    // console.log('test context3: ', updateProfile)

    // проверка, является ли пользователь моим подписчиком (нужна, потому что эта карта дергается для модалки)
    const myFollowings = useSelector(state => state?.promise?.AboutMe?.payload?.following)
    const isFollowing = myFollowings && myFollowings.some(item => item._id === data._id)

    // переход на аккаунт
    function toAccount() {
        history.push(`user/${data._id}`)
    }

    // функция подписки/отписки
    function subscribing() {
        const newData = ({
            ...updateProfile, following: [...updateProfile.following, updateProfile.following.push({ _id: data._id })]
        })

        newData && dispatch(actionFullUpdateProfile(updateProfile))
    }

    return (
        <CardHeader
            avatar={
                <Avatar
                    src={url + data?.avatar?.url}
                    aria-label="recipe"
                    sx={{ width: 36, height: 36 }}
                />
            }

            action={
                <Button
                    variant="text"
                    size="small"
                    onClick={subscribing}
                >
                    Подписаться
                </Button>
            }

            title={
                <Typography
                    sx={{
                        cursor: 'pointer',
                        width: 'fit-content'
                    }}
                    variant="subtitle2"
                    color='text.secondary'
                    onClick={toAccount}
                >
                    {data?.login || 'Anonim'}
                </Typography>
            }
        />
    )
}


// Блок с рекомендациями
function Recommended() {

    const followers = useSelector(state => state?.promise?.FindFollowers?.payload)
    const followings = useSelector(state => state?.promise?.FindFollowings?.payload)
    const myFollowings = useSelector(state => state?.promise?.AboutMe?.payload?.following)

    let recommendedList = []

    // собираем список подписок и подписчиков моих подписок и подписчико и удаляем из него повторы и моих подписок
    if (followers && followings && myFollowings) {
        const joinArr = [
            ...followers,
            ...followings.filter((item) =>
                !followers.some((current) => current._id === item._id)
            )
        ].filter((item) =>
            !myFollowings.some((current) => current._id === item._id)
        )

        // теперь оставляем в массиве только random рандомных элементов элементов
        const random = 10
        for (let i = 0; i < random; i++) {
            const randomIndex = Math.floor(Math.random() * joinArr.length)
            recommendedList.push(joinArr[randomIndex])

            // удаляем элементы из массива, чтобы снова их не показывать
            joinArr.splice(randomIndex, 1)
        }
    }


    return (
        <Box
            sx={{
                marginTop: '10px'
            }}>
            <Box
                sx={{
                    padding: '0 24px'
                }}>
                <Typography
                    variant="subtitle1"
                    color='text.secondary'
                    gutterBottom
                >
                    Рекомендации для Вас
                </Typography>
            </Box>

            <Box
                className='recommendedBox'
                sx={{
                    margin: '8px 0',
                    width: '95%',
                    paddingLeft: '8px',
                    height: '300px',
                    overflowY: 'auto'
                }}>
                {recommendedList && recommendedList.map(item => <RecommendedCard key={item._id} data={item} />)}
            </Box>
        </Box>
    )
}


export default Recommended